/*
   This file is part of Fennix Kernel.

   Fennix Kernel is free software: you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation, either version 3 of
   the License, or (at your option) any later version.

   Fennix Kernel is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Fennix Kernel. If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef __FENNIX_KERNEL_UNWIND_H__
#define __FENNIX_KERNEL_UNWIND_H__

#include <types.h>

enum _Unwind_Reason_Code
{
	_URC_NO_REASON = 0,
	_URC_FOREIGN_EXCEPTION_CAUGHT = 1,
	_URC_FATAL_PHASE2_ERROR = 2,
	_URC_FATAL_PHASE1_ERROR = 3,
	_URC_NORMAL_STOP = 4,
	_URC_END_OF_STACK = 5,
	_URC_HANDLER_FOUND = 6,
	_URC_INSTALL_CONTEXT = 7,
	_URC_CONTINUE_UNWIND = 8
};

typedef void *_Unwind_Context_Reg_Val;
typedef unsigned _Unwind_Exception_Class __attribute__((__mode__(__DI__)));
typedef unsigned _Unwind_Ptr __attribute__((__mode__(__pointer__)));
typedef unsigned _Unwind_Word __attribute__((__mode__(__unwind_word__)));
typedef signed _Unwind_Sword __attribute__((__mode__(__unwind_word__)));
typedef int _Unwind_Action;

enum _UA : _Unwind_Action
{
	_UA_SEARCH_PHASE = 1,
	_UA_CLEANUP_PHASE = 2,
	_UA_HANDLER_FRAME = 4,
	_UA_FORCE_UNWIND = 8,
	_UA_END_OF_STACK = 16
};

typedef void (*_Unwind_Exception_Cleanup_Fn)(_Unwind_Reason_Code,
											 struct _Unwind_Exception *);

typedef _Unwind_Reason_Code (*_Unwind_Personality_Fn)(int,
													  _Unwind_Action,
													  _Unwind_Exception_Class,
													  struct _Unwind_Exception *,
													  struct _Unwind_Context *);

struct _Unwind_Exception
{
	_Unwind_Exception_Class exception_class;
	_Unwind_Exception_Cleanup_Fn exception_cleanup;
	_Unwind_Word private_1;
	_Unwind_Word private_2;
} __attribute__((__aligned__));

struct _Unwind_Context;

struct _Unwind_FrameState
{
	_Unwind_Personality_Fn personality;
};

EXTERNC _Unwind_Reason_Code _Unwind_RaiseException(_Unwind_Exception *Exception);

#endif // !__FENNIX_KERNEL_UNWIND_H__
